// Loads bitmaps
#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <mem.h>

//#define PALETTE_INDEX       0x03c8
//#define PALETTE_DATA        0x03c9
//#define INPUT_STATUS        0x03da
//#define VRETRACE            0x08

//#define SCREEN_WIDTH        320       /* width in pixels of mode 0x13 */
//#define SCREEN_HEIGHT       200       /* height in pixels of mode 0x13 */
//#define NUM_COLORS          256       /* number of colors in mode 0x13 */
/* macro to skip bytes in a file */
#define fskip(fp,n)   \
{                     \
   int i;             \
		      \
   for (i=0;i<n;i++)  \
      fgetc(fp);      \
}
typedef unsigned char  byte;
typedef unsigned short word;
typedef unsigned long  dword;

typedef struct tagBITMAP              /* the structure for a bitmap. */
{
  word width;
  word height;
  byte palette[256*3];
  byte *data;
} BITMAP;
byte *VGAA=(byte *)0xA0000000L;        /* this points to video memory. */
/**************************************************************************
 *  load_bmp                                                              *
 *    Loads a bitmap file into memory.                                    *
 **************************************************************************/

void load_bmp(char *file,BITMAP *b)
{
  FILE *fp;
  long index;
  int x,Num;
  Num=256;
  /* open the file */
  if ((fp = fopen(file,"rb")) == NULL)
  {
    printf("Error opening file %s.\n",file);
    exit(1);
  }

  /* check to see if it is a valid bitmap file */
  if (fgetc(fp)!='B' || fgetc(fp)!='M')
  {
    fclose(fp);
    printf("%s is not a bitmap file.\n",file);
    exit(1);
  }

  /* read in the width and height of the image, and the
     number of colors used; ignore the rest */
  fskip(fp,16);
  fread(&b->width, sizeof(word), 1, fp);
  fskip(fp,2);
  fread(&b->height,sizeof(word), 1, fp);
  fskip(fp,22);
  fread(&Num,sizeof(word), 1, fp);
  fskip(fp,6);

  /* assume we are working with an 8-bit file */
  if (Num==0) Num=256;


  /* try to allocate memory */
  if ((b->data = (byte *) malloc((word)(b->width*b->height))) == NULL)
  {
    fclose(fp);
    printf("Error allocating memory for file %s.\n",file);
    exit(1);
  }

  /* read the palette information */
  for(index=0;index<256;index++)
  {
    b->palette[(int)(index*3+2)] = fgetc(fp) >> 2;
    b->palette[(int)(index*3+1)] = fgetc(fp) >> 2;
    b->palette[(int)(index*3+0)] = fgetc(fp) >> 2;
    x=fgetc(fp);
  }

  /* read the bitmap */
  for(index=(b->height-1)*b->width;index>=0;index-=b->width)
    for(x=0;x<b->width;x++)
      b->data[(word)(index+x)]=(byte)fgetc(fp);

  fclose(fp);
}

/**************************************************************************
 *  draw_bitmap                                                           *
 *    Draws a bitmap.                                                     *
 **************************************************************************/

void draw_bitmap(BITMAP *bmp,int x,int y)
{
  int j;
  word screen_offset = (y<<8)+(y<<6)+x;
  word bitmap_offset = 0;

  for(j=0;j<bmp->height;j++)
  {
    memcpy(&VGAA[screen_offset],&bmp->data[bitmap_offset],bmp->width);
    bitmap_offset+=bmp->width;
    screen_offset+=320;
  }
}

/**************************************************************************
 *  set_palette                                                           *
 *    Sets all 256 colors of the palette.                                 *
 **************************************************************************/

void set_palette(byte *palette)
{
  int i;

  outp(0x03c8,0);              /* tell the VGA that palette data
					 is coming. */
  for(i=0;i<256*3;i++)
    outp(0x03c9,palette[i]);    /* write the data */
}

/**************************************************************************
 *  rotate_palette                                                        *
 *    Rotates the colors of the palette.                                  *
 **************************************************************************/

void rotate_palette(byte *palette)
{
  int i,red,green,blue;

  red  = palette[3];
  green= palette[4];
  blue = palette[5];

  for(i=3;i<256*3-3;i++)
    palette[i]=palette[i+3];

  palette[256*3-3]=red;
  palette[256*3-2]=green;
  palette[256*3-1]=blue;

  set_palette(palette);
}


