/* From the book PC Game Programming Explorer
/* control.c input device routines(keyboard, joystick)*/

/* header files */
#include <assert.h>
#include <dos.h>
#include <stdio.h>
#include "control.h"
/* constans */
#define JOYSTICK_PORT (0x201)
#define PIC		(0x20)
#define NONSPECIFIC_EOI	(0x20)
#define MAX_SCANCODES	  (128)
#define NEW_HANDLER	  (1)
#define OLD_HANDLER	  (0)
#define KEYBOARD_INPUT	  (0x60)
#define KEYBOARD_XT_CLEAR (0x61)
/* module variables */
static void far interrupt (*OldInt9Handler)();  /*old key int handler*/
static int      KeyboardStateFlag = OLD_HANDLER;/* current key handler*/
static unsigned char    KeyStateArray[MAX_SCANCODES];   /*current key state */
static unsigned char    WasPressedArray[MAX_SCANCODES]; /* flags set if key hit */
/*
Key input handler, replaces bios
*/
static void far interrupt KeyIntHandler(void)
{
unsigned char ScanCode;
unsigned char Temp;
/* read scan codes */
ScanCode = inportb(KEYBOARD_INPUT);
/* clear keyboard controller on XTs */
Temp = inportb(KEYBOARD_XT_CLEAR);
outport(KEYBOARD_XT_CLEAR, Temp | 0x80); /* this line could be screwed, in book it gave odd char after Temp ? */
outport(KEYBOARD_XT_CLEAR, Temp & 0x7F);

if (ScanCode & 0x80) {
   /*Key up */
   ScanCode &= 0x7F;
   KeyStateArray[ScanCode] = 0;
}
else {
   /*Key Down */
   KeyStateArray[ScanCode] = 1;
   WasPressedArray[ScanCode] = 1;
   }
  outportb(PIC, NONSPECIFIC_EOI);
}
/*
Function: SetButtonKeysMode
sets keyboard as a set of buttons
*/
void SetButtonKeysMode(void)
{
  int i;
  assert(KeyboardStateFlag == OLD_HANDLER);
  /* init state arrays */
  for (i=0; i <128; i++) {
      KeyStateArray[i] = WasPressedArray[i] = 0;
  }
  /* install new handler */
  OldInt9Handler = getvect(9);
  setvect(9, KeyIntHandler);
  /* mark new handler as being installed */
  KeyboardStateFlag = NEW_HANDLER;
}
/* SetNormalKeysMode
sets handler back to old routine
*/
void SetNormalKeysMode(void)
{
	assert(KeyboardStateFlag == NEW_HANDLER);
	/* reinstall original handler */
	setvect(9, OldInt9Handler);
	/* indicate that the old handler is current */
	KeyboardStateFlag = OLD_HANDLER;
}

/* Function: GetKeyState
    returns 1 if key is currently down or was pressed, 0 otherwise
*/

int GetKeyState(int ScanCode)
{
     int result;
     /* scan codes should be 0-127 */
     assert(ScanCode < 128);
     assert(KeyboardStateFlag == NEW_HANDLER);

     result = KeyStateArray[ScanCode] | WasPressedArray[ScanCode];
     WasPressedArray[ScanCode] = 0;
     return result;
}
/* Function GetTrueKeyState
returns 1 if key is currently down
*/
int GetTrueKeyState(int ScanCode)
{
   int result;

   /* scan codes should be 0-127 */
   assert(ScanCode < 128);
   assert(KeyboardStateFlag == NEW_HANDLER);

   result = KeyStateArray[ScanCode];
   return result;
}
/*routines for reading state of joystick */
/* SenseJoysticks
deterines which axes are available
repeatedly reads joystick port for long time
axes that time out before done reading are assumed to be present
on a fast machine, the routine could be fooled
*/
void SenseJoysticks(JOYSTICK_STATE * JsState)
{
  register unsigned char portval;
  unsigned int limit;
  /* initialize counters */
  JsState->JsAxisAX = JsState->JsAxisAY = 0;
  JsState->JsAxisBX = JsState->JsAxisBY = 0;
  limit = 0xFFFF;
  /* don't let interrupts happen while reading */
  asm cli;
  /* start timers - date value of write is unimportant */
  outportb(JOYSTICK_PORT, 0x00);
  /* do until interesting bits all drop to zero */
  while (limit) {
    portval = inportb(JOYSTICK_PORT);
    JsState->JsAxisAX += portval & 1;
    portval >>= 1;
    JsState->JsAxisAY += portval & 1;
    portval >>= 1;
    JsState->JsAxisBX += portval & 1;
    portval >>= 1;
    JsState->JsAxisBY += portval & 1;
    limit--;
  }
  /* enable interrupts */
  asm sti;
  JsState->JsMask = 0;
  if ((JsState->JsAxisAX != 0xFFFF) && (JsState->JsAxisAX != 0)) JsState->JsMask |= 0x1;
  if ((JsState->JsAxisAY != 0xFFFF) && (JsState->JsAxisAY != 0)) JsState->JsMask |= 0x2;
  if ((JsState->JsAxisBX != 0xFFFF) && (JsState->JsAxisBX != 0)) JsState->JsMask |= 0x3;
  if ((JsState->JsAxisBY != 0xFFFF) && (JsState->JsAxisBY != 0)) JsState->JsMask |= 0x4;
}
/* ReadJoysticks
reads current values of joystick positions and button states
values in variables JsAxisAX/AY/BX/BY,  buttons JsButtonA1/A2/B1/B2
*/
void ReadJoysticks(JOYSTICK_STATE * JsState)
{
 register unsigned char portval;
 unsigned int limit;
 /* initialize counters */
 JsState->JsAxisAX = JsState->JsAxisAY = 0;
 JsState->JsAxisBX = JsState->JsAxisBY = 0;
 limit = 0xFFFF;
 /* don't let interrupts happen while we're reading */
 asm cli;
 /* start timers - date value of write is unimportant */
 outportb(JOYSTICK_PORT, 0x00);
 /* do until interesting bits all drop to zero */
 while (((portval = inportb(JOYSTICK_PORT)) & JsState->JsMask) && limit) {
     JsState->JsAxisAX += portval & 1;
     portval >>=1;
     JsState->JsAxisAY += portval & 1;
     portval >>=1;
     JsState->JsAxisBX += portval & 1;
     portval >>=1;
     JsState->JsAxisBY += portval & 1;
     limit--;
     }
     /* recognize interrupts again */
     asm sti;
     /* update button variables - note these are inverted from the port */
     (portval & 0x10) ? (JsState->JsButtonA1 = 0) : (JsState->JsButtonA1 = 1);
     (portval & 0x20) ? (JsState->JsButtonA2 = 0) : (JsState->JsButtonA2 = 1);
     (portval & 0x40) ? (JsState->JsButtonB1 = 0) : (JsState->JsButtonB1 = 1);
     (portval & 0x80) ? (JsState->JsButtonB2 = 0) : (JsState->JsButtonB2 = 1);
}
